/*
              LTR43.
            ,    
           :
    ltr43_read_write  slot crate_serial srvip
    :
        slot         -   ( 1  16)
        crate_serial -    (   )
        srvip        - ip-  ltrd/LtrServer (     )
     :    ,       

        ,   1  2  ,  3  4  .
                  .
            .

      VisualStudio:
          Visual Studio,     
    ( (Project) ->  (Properties) ->   (Configuration Properties)
    -> /++ ->  (General) ->    (Additional Include Directories))
            (ltr43api.h    
         ltr/include   )
        .lib   <  >/lib/msvc
    ( (Project) ->  (Properties) ->   (Configuration Properties) ->
     (Linker) ->  (General) ->    (Additional Library Directories)).

    !:      Visual Studio    
       ,    :
      (File) ->    (Advanced Save Options)...
        (Encoding)   (UTF8,  )/Unicode (UTF-8 with signature)
        .    ,   
           ( Consolas).
*/


#include <stdio.h>
#include <stdlib.h>
#ifdef _WIN32
#include <locale.h>
#endif

#include "ltr/include/ltr43api.h"


typedef struct {
    int slot;
    const char *serial;
    DWORD addr;
} t_open_param;

/*    .   ,  
 *   :
 * 1  -   ( 1  16)
 * 2  -   
 * 3  - ip-  */
static int f_get_params(int argc, char** argv, t_open_param* par) {
    int err = 0;
    par->slot = LTR_CC_CHNUM_MODULE1;
    par->serial = "";
    par->addr = LTRD_ADDR_DEFAULT;

    if (argc > 1)
        par->slot = atoi(argv[1]);
    if (argc > 2)
        par->serial = argv[2];
    if (argc > 3) {
        int a[4],i;
        if (sscanf(argv[3], "%d.%d.%d.%d", &a[0], &a[1], &a[2], &a[3])!=4) {
            fprintf(stderr, "  IP-!!\n");
            err = -1;
        }

        for (i=0; (i < 4) && !err; i++) {
            if ((a[i]<0) || (a[i] > 255)) {
                fprintf(stderr, " IP-!!\n");
                err = -1;
            }
        }

        if (!err) {
            par->addr = (a[0] << 24) | (a[1]<<16) | (a[2]<<8) | a[3];
        }
    }
    return err;
}
/*------------------------------------------------------------------------------------------------*/
int main(int argc, char *argv[]) {
    INT err = LTR_OK;
    TLTR43 hltr43;
    t_open_param par;

#ifdef _WIN32
    /*         Windows*/
    setlocale(LC_ALL, "");
#endif

    err = f_get_params(argc, argv, &par);

    if (err == LTR_OK) {
        LTR43_Init(&hltr43);
        /*     */
        err = LTR43_Open(&hltr43, par.addr, LTRD_PORT_DEFAULT, par.serial, par.slot);
        if (err != LTR_OK) {
            fprintf(stderr, "     .  %d (%s)\n",
                    err, LTR43_GetErrorString(err));
        } else {
            /*     */
            printf("  :\n");
            printf("  :        %s\n", hltr43.ModuleInfo.Name);
            printf("   :  %s\n", hltr43.ModuleInfo.Serial);
            printf("   : %s\n", hltr43.ModuleInfo.FirmwareVersion);
            printf("   :   %s\n", hltr43.ModuleInfo.FirmwareDate);


            /*   (    3-4  ,  1-2 -   */
            hltr43.IO_Ports.Port1 = LTR43_PORT_DIR_OUT;
            hltr43.IO_Ports.Port2 = LTR43_PORT_DIR_OUT;
            hltr43.IO_Ports.Port3 = LTR43_PORT_DIR_IN;
            hltr43.IO_Ports.Port4 = LTR43_PORT_DIR_IN;
            err = LTR43_Config(&hltr43);
            if (err != LTR_OK) {
                fprintf(stderr, "   .  %d (%s)\n",
                        err, LTR43_GetErrorString(err));
            }
        }

        if (err == LTR_OK) {
            /*   . ..      1  2 ,
             *      16  */
            DWORD wrd = 0x00005A42;
            err = LTR43_WritePort(&hltr43, wrd);
            if (err != LTR_OK) {
                fprintf(stderr, "     .  %d (%s)\n",
                        err, LTR43_GetErrorString(err));
            } else {
                printf("  !\n");
            }
        }

        if (err == LTR_OK) {
            DWORD wrd;
            /*       3  4 ,   
             *   16  */
            err = LTR43_ReadPort(&hltr43, &wrd);
            if (err != LTR_OK) {
                fprintf(stderr, "     .  %d (%s)\n",
                        err, LTR43_GetErrorString(err));
            } else {
                DWORD msk;
                printf("   ( 31-16): ");
                for (msk = (1 << 31); msk != (1 << 15); msk>>=1)
                    printf("%d ", wrd & msk ? 1 : 0);
                printf("\n");
            }
        }

        if (LTR43_IsOpened(&hltr43) == LTR_OK)
            LTR43_Close(&hltr43);
    }


    return err;
}
